#include "metropolishistogrambuilder.h"
#include "function.h"
#include "randomnumbergenerator.h"
#include <stdlib.h>
#include <iostream>
#include <cmath>

using namespace meanwalker;

// WARNING: This is an insanely BAD random number generator. For real
//          applications, use a decent one (e.g. the Mersenne twister,
//          available in the GNU Scientific Library)

class MyRandomNumberGenerator : public RandomNumberGenerator
{
public:
	MyRandomNumberGenerator()	{ }
	~MyRandomNumberGenerator()	{ }

	double pickRandomNumber()
	{
		double randomNumber = (1.0*((double)rand()/(RAND_MAX + 1.0)));

		return randomNumber;
	}
};

// This is the logarithm of the probability density we're going to
// sample: a gaussian distribution with specific mean and standard
// deviation

class LogGaussianFunction : public Function
{
public:
	LogGaussianFunction(double center, double sigma)
	{
		m_center = center;
		m_sigma = sigma;
	}

	size_t getDimension() const
	{
		return 1;
	}

	double operator()(const double *pCoords) const
	{
		double x = pCoords[0];
		double diff = (x-m_center)/m_sigma;
		return -diff*diff/2.0;
	}

	bool isLog() const
	{
		return true;
	}
private:
	double m_center, m_sigma;
};

int main(void)
{
	MyRandomNumberGenerator rndGen;
	MetropolisHistogramBuilder builder1(&rndGen);
	MetropolisHistogramBuilder builder2(&rndGen);
	LogGaussianFunction f(10, 3);

	double startCoord[] = { 8 };
	double minc[] = { 5 };
	double maxc[] = { 15 };
	size_t numBins = 30;

	// This builder is initialized with the parameter keepInside set to false.
	// This means that the sampler is allowed to go beyond the boundaries
	// specified by minc and maxc and sample the entire gaussian distribution.
	// In this case, the minc and maxc coordinates are just used to set the
	// step scale in the random walk.

	if (!builder1.init(&f, minc, maxc, startCoord, numBins, false))
	{
		std::cerr << builder1.getErrorString() << std::endl;
		return -1;
	}

	builder1.run(10000, 5000000);
	builder1.printHistograms();

	// When the keepInside flag is set to true, the walker is forced to remain
	// within the boundaries. Since the entire distribution is then contained
	// within these boundaries, the plot of the probability distribution will
	// lie a bit higher than the previous one.

	if (!builder2.init(&f, minc, maxc, startCoord, numBins, true))
	{
		std::cerr << builder2.getErrorString() << std::endl;
		return -1;
	}

	builder2.run(10000, 5000000);
	builder2.printHistograms();

	return 0;
}
