/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file function.h
 */

#ifndef MEANWALKER_FUNCTION_H

#define MEANWALKER_FUNCTION_H

#include "meanwalkerconfig.h"
#include <sys/types.h>

namespace meanwalker
{

/** Abstract base class for a (multidimensional) function. */
class MEANWALKER_IMPORTEXPORT Function
{
public:
	Function();
	virtual ~Function();

	/** A derived class must calculate the function value for the specified coordinates. */
	virtual double operator()(const double *pCoords) const = 0;

	/** Must return the dimension of the function, i.e. how many coordinates the function
	 *  needs to evaluate its value. */
	virtual size_t getDimension() const = 0;

	/** Should return true is the coordinates lie in the valid region of the probability
	 *  density, false otherwise. */
	virtual bool isValid(const double *pCoords) const 						{ return true; }

	/** Should return false if this function represents the 'normal' probability density,
	 *  or true if it represents the logarithm of the density. */
	virtual bool isLog() const = 0;
};

} // end namespace

#endif // MEANWALKER_FUNCTION_H
