#ifndef MEANWALKER_WALKER_H

#define MEANWALKER_WALKER_H

#include "meanwalkerconfig.h"
#include "function.h"
#include <errut/errorbase.h>

namespace meanwalker
{

class RandomNumberGenerator;

class Walker : public errut::ErrorBase
{
public:
	Walker(RandomNumberGenerator *pRng);
	~Walker();

	/** Sets the function to be sampled (may not be needed by an implementation) */
	void setProbabilityFunction(Function *pProbFunction)						{ m_pProbFunction = pProbFunction; }

	/** Returns the dimension of the function which is being sampled from. */
	virtual size_t getDimension() const								{ if (m_pProbFunction) return m_pProbFunction->getDimension(); return 0; }

	/** Advance the algortithm a specified number of steps. */
	virtual void walk(size_t numSteps) = 0;

	/** Returns the coordinates with the highest probability that were encountered till now during the procedure 
	 *  (returns null if not implemented or not available). */
	virtual const double *getBestCoordinates() const						{ return 0; }
protected:
	/** This function is called for each sample generated by the walkers, override it to do something
	 *  useful with it. */
	virtual void onNewSample(double functionValue, const double *pCoords)				{ }

	RandomNumberGenerator *getRandomNumberGenerator()						{ return m_pRndGen; }
	Function *getProbabilityFunction()								{ return m_pProbFunction; }
private:
	RandomNumberGenerator * const m_pRndGen;
	Function *m_pProbFunction;
};

} // end namespace   


#endif // MEANWALKER_WALKER_H

